
#ifndef qobject_hookH
#define qobject_hookH

#include <qobject.h>
#include "qtypes.h"


class QObject_hook : public QObject {
  Q_OBJECT
  public:
    QObject_hook(QObject *handle) : QObject(NULL) {
      this->handle = handle;
      this->events.func = NULL;
      this->destroyed_event.func = NULL;
    }

    virtual ~QObject_hook() {
      if ( handle )
      {
        handle->removeEventFilter( this );
	handle->disconnect( this );
	handle = NULL;
      }
    }

    void hook_events(QHook &hook) {
	if ( !events.func ) {
	  handle->installEventFilter(this);
	  events = hook; }
	if (!hook.func)
	  handle->removeEventFilter(this);
	events = hook;
      }

    void hook_destroyed(QHook &hook) {
      if ( !destroyed_event.func )
        connect(handle, SIGNAL(destroyed()), this, SLOT(destroyed_hook()));
      destroyed_event = hook;
      if ( !hook.func )
        disconnect(handle, SIGNAL(destroyed()), this, SLOT(destroyed_hook()));
    }

  protected:
    QObject *handle;
	  virtual bool eventFilter(QObject *sender, QEvent *event) {
	    if ( events.func ) {
		typedef bool (*func_type)(void *data, QObject *sender, QEvent *event);
		return (*(func_type)events.func)(events.data, sender, event);
	    }
	    return false;
	  }
  
  private slots:
    void destroyed_hook() {
      if ( destroyed_event.func ) {
        typedef void (*func_type)(void *data);
        (*(func_type)destroyed_event.func)(destroyed_event.data);
      }
    }
  
  private:
    QHook events;
    QHook destroyed_event;
};

#endif
